#include "ppc.h"
#include "framebuffer.h"
#include <math.h>
#include <float.h>
#include <iostream>



PPC::PPC(int _w, int _h, float hfov) : w(_w), h(_h) {

  C = V3(0.0f, 0.0f, 0.0f);
  a = V3(1.0f, 0.0f, 0.0f);
  b = V3(0.0f, -1.0f, 0.0f);
  c[0] = -(float)w/2.0f;
  c[1] = +(float)h/2.0f;
  float tan2 = tanf(hfov/180.0f*3.1415f/2.0f);
  c[2] = -(float)w/2.0f/tan2;

}

bool PPC::Project(V3 P, V3& projP) {

  M3x3 cam;
  cam.SetColumn(0, a);
  cam.SetColumn(1, b);
  cam.SetColumn(2, c);

  V3 q = cam.Invert()*(P-C);

  projP[0] = FLT_MAX;
  if (q[2] <= 0.0f)
    return false;

  projP[0] = q[0] / q[2];
  projP[1] = q[1] / q[2];
  projP[2] = 1.0f / q[2];

  if (projP[0] < 0.0f || projP[0] > (float) w
    || projP[1] < 0.0f || projP[1] > (float) h)
    return false;

  return true;

}

void PPC::TranslateRight(float tstep) {

  C = C + a.Normalized()*tstep;

}

void PPC::TranslateUp(float tstep) {

  C = C - b.Normalized()*tstep;

}

void PPC::TranslateForward(float tstep) {

  C = C + GetVD()*tstep;

}

V3 PPC::GetVD() {

  return (a^b).Normalized();

}

void PPC::ZoomIn(float f) {

  float fl = GetF();
  float newfl = fl * f;
  float tstep = newfl-fl;
  c = c + GetVD()*tstep;

}

float PPC::GetF() {

  return c*GetVD();

}


PPC PPC::Interpolate(PPC* ppcn, int fi, int fN) {

  PPC ret(*this);
  float frac = (float) fi / (float) fN;
  ret.C = C + (ppcn->C - C)*frac;

  V3 vd0 = GetVD();
  V3 vdn = ppcn->GetVD();
  if ((vd0*vdn) < 0.0f) {
    cerr << "INFO: angle between view directions > 90" << endl;
    return ret;
  }
  V3 vdk = (vd0 + (vdn-vd0)*frac).Normalized();
  V3 v = b + (ppcn->b-b)*frac;

  ret.a = (v^vdk).Normalized()*a.Length();
  ret.b = (vdk^ret.a).Normalized()*b.Length();
  ret.c = vdk*GetF() - ret.b*(float)h/2.0f - ret.a*(float)w/2.0f;
  return ret;

}

void PPC::RotateAboutAxisThroughC(V3 axis, float rstep) {

  a = a.RotateDirectionAboutArbitraryAxis(axis, rstep);
  b = b.RotateDirectionAboutArbitraryAxis(axis, rstep);
  c = c.RotateDirectionAboutArbitraryAxis(axis, rstep);

}